/*
 *  sudoku_lib.c
 *  simu_anneal
 *
 *  Created by wei sun on 2/13/06.
 *  Copyright 2006 __MyCompanyName__. All rights reserved.
 *
 */
#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include "nrutil.h"
#include "common_lib.h"
#include "sudoku_lib.h"

/* ---------------------------------------------------------
 * permute a integer vector
 * ---------------------------------------------------------
 */

void permute(int* v, int* u, int nl, int nh) {

	int i, t, k;
	long idum=(-113);
	
	for (i=nl; i<=nh; i++) { u[i] = v[i]; }
	for (i=nl; i<=nh; i++) {
		// generate a random integer from i to nh
		t = floor(ran3(&idum)*(nh-i+1)) + i;
		k = u[t];
		u[t] = u[i];
		u[i] = k;
	}
}

/* ---------------------------------------------------------
 * fill a sudoku matrix by random permuation, 
 * while keeping the given value
 *
 * assume that dm and im are 9*9 matrix
 * ---------------------------------------------------------
 */

void fill_sudoku(int **dm){
	int i, j, k, t, N=9;
	int *v, *u;
	v = ivector(1,N);
	u = ivector(1,N);
	for(i=1; i<=N; i++){
		for(k=1; k<=N; k++){ v[k] = k; }
		for(j=1; j<=N; j++){
			if(dm[i][j] > 0){ v[dm[i][j]] = 0; }
		}
		permute(v, u, 1, N);
		t = 1;
		for(j=1; j<=N; j++){
			if(dm[i][j] == 0){
				while(u[t]==0){ t++; }
				dm[i][j] = u[t];
				t++;
			}
		}
	}
	free_ivector(v, 1, N);
	free_ivector(u, 1, N);
}



/* ---------------------------------------------------------
 * calculate the penalty of one row
 * ---------------------------------------------------------
 */

int cPRow(int r, int penalty, int **dm){
	int j, k, newP, N=9;
	int *v;
	v = ivector(1,N);
	newP = 0;
	for(k=1;k<=N;k++){ v[k] = -1; }
	for(j=1;j<=N;j++){ v[dm[r][j]]++; }
	for(k=1;k<=N;k++){ if(v[k]>0) newP+=v[k]*penalty; }
	return(newP);
}

/* ---------------------------------------------------------
 * calculate the penalty of one col
 * ---------------------------------------------------------
 */

int cPCol(int c, int penalty, int **dm){
	int i, k, newP, N=9;
	int *v;
	v = ivector(1,N);
	newP = 0;
	for(k=1;k<=N;k++){ v[k] = -1; }
	for(i=1;i<=N;i++){ v[dm[i][c]]++; }
	for(k=1;k<=N;k++){ if(v[k]>0) newP+=v[k]*penalty; }
	return(newP);
}

/* ---------------------------------------------------------
 * calculate the penalty of one subgrid
 *
 * the way to order the subgrids, i.e. s=
	0	1	2
	3	4	5
	6	7	8
 * ---------------------------------------------------------
*/
int cPSg(int s, int penalty, int **dm){
	int i0, j0, i, j, k, newP, N=9;
	int *v;
	v = ivector(1,N);
	newP = 0;
	
	i0=s/3; j0=s%3; // s = i0*3 + j0
	for(k=1;k<=N;k++){ v[k] = -1; }
	for(i=i0*3+1; i<=i0*3+3; i++){
		for(j=j0*3+1; j<=j0*3+3; j++){ v[dm[i][j]]++; }
	}
	for(k=1;k<=N;k++){ if(v[k]>0) newP+=v[k]*penalty; }
	return(newP);
}

/* ---------------------------------------------------------
 * calculate the change of penalty 
 * assume that dm is 9*9 matrix
 * ---------------------------------------------------------
 */

int pChange(int r1, int c1, int r2, int c2, int **dm, int *prows, int *pcols, 
int *psgs, int prow, int pcol, int psg, int propose){

	int tmp, N=9, newP1, newP2, ed, s1, s2;
	int *v;
	v = ivector(1,N);
	
	tmp = dm[r1][c1];
	dm[r1][c1] = dm[r2][c2];
	dm[r2][c2] = tmp;
	
	ed = 0;
	newP1 = cPCol(c1, pcol, dm);
	newP2 = cPCol(c2, pcol, dm);
	ed += newP1 + newP2 - pcols[c1] - pcols[c2];
	if(propose == 0){ pcols[c1] = newP1; pcols[c2] = newP2; }
	
	newP1 = cPRow(r1, prow, dm);
	newP2 = cPRow(r2, prow, dm);
	ed += newP1 + newP2 - prows[r1] - prows[r2];
	if(propose == 0){ prows[r1] = newP1; prows[r2] = newP2; }
	
	
	// check whether (r1,c1) and (r2,c2) belongs to the same subgrid
	s1 = ((r1-1)/3)*3 + (c1-1)/3;
	s2 = ((r2-1)/3)*3 + (c2-1)/3;
	
	if(s1 != s2){
		newP1 = cPSg(s1, psg, dm);
		newP2 = cPSg(s2, psg, dm);
		ed += newP1 + newP2 - psgs[s1] - psgs[s2];
		if(propose == 0){ psgs[s1] = newP1; psgs[s2] = newP2; }
	}

	if(propose == 1){
		tmp = dm[r1][c1];
		dm[r1][c1] = dm[r2][c2];
		dm[r2][c2] = tmp;
	}
	
	return(ed);
}

/* ---------------------------------------------------------
 * metroplis 
 * ---------------------------------------------------------
 */

int metrop(int de, float t){
	static long gljdum=-19;
	return de < 0 || ran3(&gljdum) < exp(-de/t);
}

/* ---------------------------------------------------------
 * generate a sudoku matrix 
 * assume that dm is 9*9 matrix
 * ---------------------------------------------------------
 */

void sudoku(int **dm, int **im){
	int i, j, k, s, r1, r2, c1, c2, ans, N=9;
	int nsucc, nover, nlimit, psg, prow, pcol, pall;
	float t, de, TFACTR=0.9;
	long idum;
	unsigned long iseed;
	int *v, *psgs, *prows, *pcols;
	
	nsucc = 0;		// total number of successes in one temperature
	nover = 500;	// maximum number of swaps
	nlimit= 50;		// maximum number of succesful swaps
	psg = 1;		// penalty for repeats in subgrids 
	prow = 2;		// penalty for repeats in one row
	pcol = 3;		// penalty for repeats in one column
	t = 1;			// temperature
	prows = ivector(1,N);	// penalty of repeats in rows
	pcols = ivector(1,N);	// penalty of repeats in cols
	psgs  = ivector(0,N-1);	// penalty of repeats in subgrids
	idum  = -19;
	iseed = 113;
	v = ivector(1,N);
	
	for(k=1;k<=N;k++){ prows[k]=0; pcols[k]=0; psgs[k]=0; }
	// first, calculate repeats in row
	for(i=1;i<=N;i++){ prows[i] = cPRow(i, prow, dm); }
	// second, calculate repeats in col
	for(j=1;j<=N;j++){ pcols[j] = cPCol(j, pcol, dm); }
	// thrid, calculate repeats in subgrids
	for(s=0;s<=N-1;s++){ psgs[s] = cPSg(s, psg, dm); }

	printf("\n penalty(rows):\n");
	for (k=1;k<=N;k++) { printf("%d\t", prows[k]); }
	printf("\n penalty(cols):\n");
	for (k=1;k<=N;k++) { printf("%d\t", pcols[k]); }
	printf("\n penalty(subgrids):\n");
	for (k=0;k<=N-1;k++) { printf("%d\t", psgs[k]); }

	// calculate the total penalty
	pall = 0;
	for(k=1;k<=N;k++){ pall+=prows[k]; pall+=pcols[k]; }
	for(k=0;k<=N-1;k++){ pall+=psgs[k]; }
	printf("original penalty is %d\n", pall);
	
	for (j=1;j<=100;j++) { // try up to 100 temperature steps
		nsucc=0;
		for (k=1;k<=nover;k++) {
			do{
				r1=ceil(ran3(&idum)*N);
				r2=ceil(ran3(&idum)*N);
				c1=ceil(ran3(&idum)*N);
				c2=ceil(ran3(&idum)*N);
			} while(im[r1][c1]==1 || im[r2][c2]==1 || (r1==r2 && c1==c2));
			
			// calculate change of penalty
			// printf("to swap (%d,%d) and (%d,%d)\n", r1, c1, r2, c2);
			de = pChange(r1, c1, r2, c2, dm, prows, pcols, psgs, prow, pcol, psg, 1);
			
			ans=metrop(de,t);
			if (ans) {
				pChange(r1, c1, r2, c2, dm, prows, pcols, psgs, prow, pcol, psg, 0);
				++nsucc;
				pall += de;
			}
			if (nsucc >= nlimit) break;
		}
		printf("\n T = %f,  Penalty = %d,  ", t, pall);
		printf("Successful Moves: %d\n", nsucc);
		t *= TFACTR;
		if (pall == 0) return;
		/*
		printf("result:\n");
		for (k=1;k<=N;k++) {
			for (s=1;s<=N;s++) { printf("%d\t", dm[k][s]); }
			printf("\n");
		}
		
		printf("\n penalty(rows):\n");
		for (k=1;k<=N;k++) { printf("%d\t", prows[k]); }
		printf("\n penalty(cols):\n");
		for (k=1;k<=N;k++) { printf("%d\t", pcols[k]); }
		printf("\n penalty(subgrids):\n");
		for (k=0;k<=N-1;k++) { printf("%d\t", psgs[k]); }
		*/
	}
	printf("\n penalty(rows):\n");
	for (k=1;k<=N;k++) { printf("%d\t", prows[k]); }
	printf("\n penalty(cols):\n");
	for (k=1;k<=N;k++) { printf("%d\t", pcols[k]); }
	printf("\n penalty(subgrids):\n");
	for (k=0;k<=N-1;k++) { printf("%d\t", psgs[k]); }

	free_ivector(v,1,N);
	free_ivector(prows,1,N);
	free_ivector(pcols,1,N);
	free_ivector(psgs,0,N-1);
}

